/*#=======================================================================
 *# Copyright (C) 1998 Recital corporation Inc.
 *# As an unpublished licenced proprietary work.
 *# All rights reserved worldwide.
 *#=======================================================================
 *
 * 	MODULE		: jdbc_test.java
 *
 * 	PURPOSE		: Recital JDBC Developer installation test program
 *
 * 	AUTHOR		: Recital Corporation
 *
 * 	DATE		: 04-Sep-1998
 *
 *======================================================================*/

//-----------------------------------------------------------------------
//-- standard imports required by JDBC Developer
//-----------------------------------------------------------------------
import java.sql.*;
import java.io.*;
import java.net.URL;
import java.math.BigDecimal;
import Recital.sql.*;


public class jdbc_test {

	public static void main(String argv[]) {
		int	i;
		ResultSet rs;
		ResultSetMetaData rsmd;

		System.out.println("Recital JDBC driver Installation verification program started.");

		for (int ii=0; ii<1; ++ii) {
		try {
			//-----------------------------------------------------------------------
			//-- Load the Client Driver for the
			//-- Recital Universal Application Server
			//-----------------------------------------------------------------------
			System.out.println("Loading Recital JDBC driver.");
			new RecitalDriver();

			//-----------------------------------------------------------------------
			//-- The standard format of the connection URL is as follows:
			//-----------------------------------------------------------------------
			String url = "jdbc:Recital:" +
					     "SERVERNAME=?;" +
					     "DATABASE=jdbc_test;" +
					     "USERNAME=?;" +
					     "PASSWORD=?;" +
					     "LOGGING=TRUE;" +
					     "LOGFILE=jdbc_test.log;" +
					     "ENCRYPTION=false";


			//-----------------------------------------------------------------------
			//-- The extended format provides connectivity to remote servers:
			//--	ODBC data sources
			//--	Oracle
			//--	Informix
			//--	Ingres
			//--	DB2/6000
			//--
			//--	String url =
			//--	"jdbc:Recital:" +
			//--		"SERVERNAME=?;" +
			//--		"DIRECTORY=c:\\Program Files\\Recital\\UAS\\NETdata;" +
			//--		"DATABASE=database;" +
			//--		"USERNAME=recital;" +
			//--		"PASSWORD=recital";
			//--	    "DEBUGGING=true;" +
			//--		"LOGGING=true"
			//--		"LOGFILE=filename"
			//--		"GATEWAY=type@node:username/password-database";
			//--		"GATEWAY=odbc:Northwind";
			//--
			//-- Key parameters:
			//--
			//--	SERVERNAME	The IP address of the Universal Application Server
			//--				? connects to the local machine
			//--	DIRECTORY	The startup directory on the server
			//--	DATABASE	The startup database on the server
			//--	USERNAME	Login username
			//--	PASSWORD	Login password
			//--	LOGGING		Turn logging on/off
			//--	LOGFILE		Where to log
			//--	GATEWAY		Gateway definition
			//--
			//-- Gateways:
			//--
			//--	Remote ODBC data sources:
			//--
			//--		GATEWAY=odbc:datasource
			//--
			//--	Remote SQL data sources:
			//--
			//--		GATEWAY=type@node:username/password-database
			//--
			//--			type can be:
			//--
			//--			ORA		Oracle
			//--			INF		Informix
			//--			ING		Ingres
			//--			DB2		DB2/6000
			//--			JDB		Server-side JDBC driver
			//--
			//--			node can be any IP address
			//-----------------------------------------------------------------------

			//-----------------------------------------------------------------------
			//-- Connect
			//-----------------------------------------------------------------------
			System.out.println("Connecting to the Recital UAS.");
			//Connection con = DriverManager.getConnection(url, "recital", "recital");
			Connection con = DriverManager.getConnection(url);

			//-----------------------------------------------------------------------
			//-- Create a statement on the connection
			//-----------------------------------------------------------------------
			Statement stmt = con.createStatement();

			//-----------------------------------------------------------------------
			//-- Query info from the driver
			//-----------------------------------------------------------------------
			DatabaseMetaData dbmd = con.getMetaData();
			System.out.println("Driver	 "+dbmd.getDriverName()+" Version "+dbmd.getDriverVersion());
			System.out.println("Database "+dbmd.getDatabaseProductName() + "\nVersion " + dbmd.getDatabaseProductVersion());

			//-----------------------------------------------------------------------
			//-- Query table information from the server
			//-----------------------------------------------------------------------
			String types[] = {"TABLE"};
			rs = dbmd.getTables(null, null, null,types);
			while (rs.next()) {
				String t3 = rs.getString("TABLE_NAME");
				System.out.println("Table : "+t3);
			}
			rs.close();

			//-----------------------------------------------------------------------
			//-- Create a few test tables on the server
			//-----------------------------------------------------------------------
			System.out.println("Create Table jdbctest.");
			stmt.execute("create table jdbctest " +
					     "(rowno char(10), "+
						 "testChar30 char(30) not null, " +
						 "testNumeric5_2 numeric(5,2)," +
						 "testDecimal9_3 decimal(9,3)," +
						 "testInteger integer," +
						 "testSmallInt smallint," +
						 "testFloat float," +
						 "testDouble double," +
						 "testDate date," +
						 "testTime time," +
						 "testTimeStamp timestamp," +
						 "testVarChar varchar(9))");
			System.out.println("Create Table jdbctest2.");
			stmt.execute("create table jdbctest2 " +
						 "(rowno char(10), "+
						 "name char(30) not null, " +
						 "testNumeric5_2 numeric(5,2)," +
						 "testDecimal9_3 decimal(9,3)," +
						 "testInteger integer," +
						 "testSmallInt smallint," +
						 "testFloat float," +
						 "testDouble double," +
						 "testDate date," +
						 "testTime time," +
						 "testTimeStamp timestamp," +
						 "testVarChar varchar(9))");

			//-----------------------------------------------------------------------
			//-- Insert some data into the tables
			//-----------------------------------------------------------------------
			System.out.println("Insert rows into jdbctest.");
			for (i=0; i<1; ++i) {
				stmt.execute("insert into jdbctest values (" +
							"'"+"".valueOf(i+1)+"',"+
							"'test1', "+
							"9.7, "+
							"12.3124, "+
							"234, "+
							"".valueOf(i)+", "+
							"12.1423234, "+
							"23.1242314, "+
							"'1996-06-18', "+
							"'11:11:11', "+
							"'19960618231159',"+
							"'blabla')");
			}
			System.out.println("Insert rows into jdbctest2.");
			for (i=0; i<100; ++i) {
				stmt.execute("insert into jdbctest2 values (" +
							"'"+"".valueOf(i+1)+"',"+
							"'" + "test"+"".valueOf(i+1) + "', "+
							"9.7, "+
							"12.3124, "+
							"234, "+
							"".valueOf(i)+", "+
							"12.1423234, "+
							"23.1242314, "+
							"'1996-06-18', "+
							"'11:11:11', "+
							"'19960618231159',"+
							"'blabla')");
			}

			//-----------------------------------------------------------------------
			//-- Update the remote data
			//-----------------------------------------------------------------------
			System.out.println("Update rows in jdbctest.");
			int nr_rows = stmt.executeUpdate("update jdbctest set testdate='1998-06-18' where TESTCHAR30 = 'test1'");
			System.out.println("" + nr_rows + " rows updated.");

			//-----------------------------------------------------------------------
			//-- Query for data
			//-----------------------------------------------------------------------
			System.out.println("Select rows from jdbctest.");
			rs = stmt.executeQuery("SELECT * from jdbctest");
			rsmd = rs.getMetaData();
			int nr_cols = rsmd.getColumnCount();
			for (i = 1; i <= nr_cols; i++) {
				System.out.println(rpad("Column[" + i + "]",10) +
								   " : " + rpad(rsmd.getColumnName(i),10) +
								   " " + rsmd.getColumnTypeName(i) +
								   "," + "".valueOf(rsmd.getPrecision(i)) +
								   "," + "".valueOf(rsmd.getScale(i))
								   );
			}
			System.out.println("Got results:");

			//-----------------------------------------------------------------------
			//-- Display the data
			//-----------------------------------------------------------------------
			i = 1;
			while (rs.next()) {
				String t1 = rs.getString(rsmd.getColumnName(2));
				BigDecimal t2 = rs.getBigDecimal("TESTNUMERIC5_2", 5);
				BigDecimal t3 = rs.getBigDecimal("TESTDECIMAL9_3", 1);
				int t4 = rs.getInt("TESTINTEGER");
				short t5 = rs.getShort("TESTSMALLINT");
				float t6 = rs.getFloat("TESTFLOAT");
				double t7 = rs.getDouble("TESTDOUBLE");
				Date t8 = rs.getDate("TESTDATE");
				Time t9 = rs.getTime("TESTTIME");
				Timestamp t10 = rs.getTimestamp("TESTTIMESTAMP");
				String t11 = rs.getString("TESTVARCHAR");
				System.out.println(("".valueOf(i)).trim()+": "+
									t1.trim() + "," +
									t2 + "," +
									t3 + "," +
									t4 + "," +
									t5 + "," +
									t6 + "," +
									t7 + "," +
									t8 + "," +
									t9 + "," +
									t10);
				++i;
			}

			//-----------------------------------------------------------------------
			//-- Query on a join
			//-----------------------------------------------------------------------
			System.out.println("Join jdbctest and jdbctest2.");
			rs = stmt.executeQuery("SELECT jdbctest.rowno as rowno, "+
									"jdbctest.testchar30 as char, "+
									"jdbctest2.name as name "+
									"from jdbctest, jdbctest2 "+
									"WHERE jdbctest.rowno = jdbctest2.rowno");
			rsmd = rs.getMetaData();
			nr_cols = rsmd.getColumnCount();
			for (i = 1; i <= nr_cols; i++) {
				System.out.println("rs  Column[" + i + "] : " +
									rsmd.getColumnName(i) + " <" +
									rsmd.getColumnTypeName(i) + ">");
			}
			System.out.println("Got results:");

			//-----------------------------------------------------------------------
			//-- Display the data
			//-----------------------------------------------------------------------
			i = 1;
			while (rs.next()) {
				String t1 = rs.getString("ROWNO");
				String t2 = rs.getString("CHAR");
				String t3 = rs.getString("NAME");
				System.out.println(("".valueOf(i)).trim()+": "+
									t1.trim() + "," +
									t2.trim() + "," +
									t3.trim());
				++i;
			}

			//-----------------------------------------------------------------------
			//-- Release the statement
			//-----------------------------------------------------------------------
			stmt.close();

			//-----------------------------------------------------------------------
			//-- Disconnect from the server
			//-----------------------------------------------------------------------
			con.close();
			System.out.println("Recital JDBC driver Installation verification program complete.");
			System.in.read();

		} catch (Exception e) {
			System.out.flush();
			System.err.flush();
			DriverManager.println("Recital JDBC driver exception: " + e.getMessage());
			e.printStackTrace();
		}
		}
	}


	static String rpad(String s, int width) {
		while (s.length() < width) s += " ";
		return s;
	}
}

