/*#============================================================================
 *# Copyright (C) 1989-1991 Recital Corporation Inc.
 *# As an unpublished licenced proprietary work.
 *# All rights reserved worldwide.
 *#============================================================================
 *
 *	MODULE		: dbapi.h
 *
 *	PURPOSE		: Recital/API header file.
 *
 *	AUTHOR		: Recital Corporation Inc.
 *
 *	DATE		: 29-may-1991
 *
 *==========================================================================*/

#ifndef RECITAL_DBAPI_H
#define RECITAL_DBAPI_H

#ifdef _WIN32
#define RECITAL_FUNCTION __declspec(dllexport) void
#define RECITAL_EXPORT __declspec(dllexport) 
#define strcasecmp stricmp
#define WINAPI __stdcall
#else
#define RECITAL_FUNCTION void
#define RECITAL_EXPORT 
#define WINAPI
#endif
#define INIT_API \
static	struct	API_FUNCTION_ADDRESS_TABLE *api_function_address_table = NULL; \
static  char	current_version[] = "9.2.0"; \
RECITAL_EXPORT char * getVersion(void) \
{ \
	return current_version; \
} \
int WINAPI initAPI(struct	API_FUNCTION_ADDRESS_TABLE *function_address_table) \
{ \
	api_function_address_table = function_address_table;  \
	return 0; \
} \
RECITAL_EXPORT  struct API_SHARED_FUNCTION_TABLE *getFunctions(void) \
{ \
	return api_function_table; \
}


/*=============================================================================
 * API constants
 *===========================================================================*/

#ifndef EXPORT
#ifdef	VAXVMS
#ifdef	VAXC
#define	EXPORT	globaldef noshare
#else
#define	EXPORT	extern
#endif
#else
#define EXPORT
#endif
#endif

#ifndef IMPORT
#ifdef	VAXC
#define	IMPORT	globalref
#else
#define	IMPORT	extern
#endif
#endif


/* data types returned by api_parinfo() */
#define	API_UTYPE	0	/* undefined				*/
#define	API_CTYPE	1	/* character				*/
#define	API_NTYPE	2	/* numeric				*/
#define	API_LTYPE	4	/* logical				*/
#define	API_DTYPE	8	/* date					*/
#define	API_MTYPE	16	/* memo				 	*/
#define	API_REFPAR	32	/* passed by reference with @		*/
#define	API_TTYPE	64	/* datetime				*/
#define	API_YTYPE	128	/* currency				*/
#define	API_OTYPE	256	/* object				*/
#define	API_ATYPE	512	/* array				*/

/* general constants */
#define	API_TOP		-1	/* DBF_GOTO( API_TOP )			*/
#define	API_BOTTOM	-2	/* DBF_GOTO( API_BOTTOM )		*/
#define	API_PUBLIC	0	/* PUBLIC memvars			*/
#define API_PRIVATE	1	/* PRIVATE memvars			*/

/* memory variable data types */
#ifndef DATE
#define DATE		unsigned int
#endif
#ifndef MEMO
#define MEMO		off_t
#endif
#ifndef TIME
#define TIME		unsigned int
#endif
#define CHARACTER	char 
#define LOGICAL		int
#define NUMERIC		double
#ifndef CURRENCY
struct  RCT_CURRENCY_DEF {
	unsigned short int data[4] ;
};
#define CURRENCY struct RCT_CURRENCY_DEF
#endif
#ifndef DATETIME
struct  RCT_DATETIME_DEF {
	DATE	date;
	TIME	time;
};

#define DATETIME struct RCT_DATETIME_DEF
#endif

#define OBJECT		char *
#define OBJARG		char *

#define	API_FUNCTION	0
#define	API_CLASS	1

/*=============================================================================
 * API data structures
 *===========================================================================*/

struct	API_FUNCTION_TABLE {
	char	*name;		/* pointer to recital function name	*/
	int	(*address)();	/* address of C support function	*/
};

struct	API_SHARED_FUNCTION_TABLE {
	char	*name;		/* pointer to recital function name	*/
	char	*apiname;	/* pointer to C support function name	*/
	int	type;		/* function type 0 function 1 class	*/
};

#ifdef errno
#undef errno
#endif

struct API_EXPRESSION {
	int	errno;		/* Recital error() number	     	*/
	char	type;		/* data type of result		     	*/
				/* C	= character or memo	     	*/
				/* N 	= numeric   			*/
				/* D	= date			     	*/
				/* L	= logical		     	*/
				/* T	= datetime		     	*/
				/* Y	= currency		     	*/
	int	width;		/* width of result		     	*/
	int	decimal;	/* decimal places in result	     	*/
	char	*character;	/* CHARACTER result		     	*/
	double	number;		/* NUMERIC result		     	*/
	char	logical;	/* LOGICAL result		     	*/
	DATE    date;		/* DATE result			     	*/
	DATETIME datetime;	/* DATETIME result			*/
	CURRENCY currency;	/* CURRENCY result			*/
};

union 	API_UNION {
	char		*info_character; /* character memory variable  	*/
	char		info_logical;    /* logical memory variable	*/
	double		info_number;     /* numeric memory variable	*/
	DATE		info_date;       /* date memory variable	*/
	DATETIME	info_datetime;	 /* datetime memory variable	*/
	CURRENCY        info_currency;   /* currency memory variable	*/
};

struct	API_MEMVAR {
	char	type;		/* data type of memory variable		*/
	union	API_UNION value;/* value of memory variable		*/
	int	width;		/* display width			*/
	int	decimal;	/* display # of decimal places		*/
};

struct	API_FIELD {
	int	fieldno;	/* field number				*/
	char	type;		/* data type of field			*/
	int	size;		/* storage size in the record		*/
	int	width;		/* display width			*/
	int	decimal;	/* number of decimal places for numerics*/
	char	*value;		/* pointer to data in current record	*/
};

/*=============================================================================
 * API level 1 functions 
 *===========================================================================*/
#define	_parc			api_function_address_table->api_par_c
#define	_pards			api_function_address_table->api_par_ds
#define	_parts			api_function_address_table->api_par_ts
#define	_parys			api_function_address_table->api_par_ys
#define	_parl			api_function_address_table->api_par_l
#define	_parni			api_function_address_table->api_par_ni
#define	_parnl			api_function_address_table->api_par_nl
#define	_parnd			api_function_address_table->api_par_nd
#define	_parclen		api_function_address_table->api_par_clen
#define _parcsiz		api_function_address_table->api_par_csiz
#define _parinfa		api_function_address_table->api_par_infa
#define _parinfo		api_function_address_table->api_par_info
#define	_paro			api_function_address_table->api_par_o
#define _parse_parameters	api_function_address_table->api_parse_parameters	/* new in 9.0.4 */
#define _exmgrab(ARG)		malloc(ARG)
#define _exmback(ARG,LEN)	free(ARG)
#define _xgrab(ARG)		malloc(ARG)
#define _xalloc(ARG)		malloc(ARG)
#define _xfree(ARG,LEN)		free(ARG)
#define	_retc(ARG)		{api_function_address_table->api_ret_c(ARG);return;}
#define _retclen(ARG,LEN)	{api_function_address_table->api_ret_clen(ARG,LEN);return;}
#define _retds(ARG)		{api_function_address_table->api_ret_ds(ARG);return;}
#define _retts(ARG)		{api_function_address_table->api_ret_ts(ARG);return;}
#define _retys(ARG)		{api_function_address_table->api_ret_ys(ARG);return;}
#define _retl(ARG)		{api_function_address_table->api_ret_l(ARG);return;}
#define _retni(ARG)		{api_function_address_table->api_ret_ni(ARG);return;}
#define _retnl(ARG)		{api_function_address_table->api_ret_nl(ARG);return;}
#define _retnd(ARG)		{api_function_address_table->api_ret_nd(ARG);return;}
#define _reto(ARG)		{api_function_address_table->api_ret_o(ARG);return;}
#define	_ret			{api_function_address_table->api_ret();return;}
#define	PCOUNT			(api_function_address_table->api_par_info(0))
#define ISCHAR(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_CTYPE)
#define ISNUM(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_NTYPE)
#define ISLOG(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_LTYPE)
#define ISDATE(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_DTYPE)
#define ISMEMO(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_MTYPE)
#define ISBYREF(ARG)		((api_function_address_table->api_par_info(ARG)&API_REFPAR)
#define ISARRAY(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_ATYPE)
#define ISDATETIME(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_TTYPE)
#define ISCURRENCY(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_YTYPE)
#define ISOBJECT(ARG)		((api_function_address_table->api_par_info(ARG)&~API_REFPAR)==API_OTYPE)
#define ALENGTH(ARG)		(api_function_address_table->api_par_infa(ARG,0))

/*=============================================================================
 * API level 2 functions 
 *===========================================================================*/
#define COMMAND			api_function_address_table->api_command
#define	EXPRESSION		api_function_address_table->api_expression
#ifdef ERROR
#undef ERROR
#endif
#define ERROR			api_function_address_table->api_error
/*=============================================================================
 * API level 3 functions 
 *===========================================================================*/

/* database functions */
#define DBF_FETCH		api_function_address_table->api_dbf_fetch
#define	DBF_READ		api_function_address_table->api_dbf_read
#define DBF_RECALL		api_function_address_table->api_dbf_recall
#define DBF_DELETE		api_function_address_table->api_dbf_delete
#define DBF_DELETED		api_function_address_table->api_dbf_isdeleted
#define DBF_UPDATE		api_function_address_table->api_dbf_update
#define DBF_APPEND		api_function_address_table->api_dbf_append
#define DBF_SEEK		api_function_address_table->api_dbf_seek
#define DBF_LOCKF		api_function_address_table->api_dbf_lockf
#define DBF_UNLOCKF		api_function_address_table->api_dbf_unlockf
#define DBF_LOCKR		api_function_address_table->api_dbf_lockr
#define DBF_UNLOCKR		api_function_address_table->api_dbf_unlockr
#define	DBF_RECNO		api_function_address_table->api_dbf_recno
#define	DBF_RECCOUNT		api_function_address_table->api_dbf_reccount
#define	DBF_RECSIZE		api_function_address_table->api_dbf_recsize
#define	DBF_RECBUFFER		api_function_address_table->api_dbf_recbuffer
#define	DBF_FILTER		api_function_address_table->api_dbf_filter
#define	DBF_DBF			api_function_address_table->api_dbf_dbf
#define DBF_ALIAS		api_function_address_table->api_dbf_alias
#define DBF_SELECT		api_function_address_table->api_dbf_select
#define	DBF_ISREADONLY		api_function_address_table->api_dbf_isread_only
#define DBF_ISEXCLUSIVE		api_function_address_table->api_dbf_isexclusive
#define	DBF_ISBOF		api_function_address_table->api_dbf_isbof
#define DBF_ISEOF		api_function_address_table->api_dbf_iseof
#define DBF_FMT			api_function_address_table->api_dbf_fmt
#define DBF_NDX			api_function_address_table->api_dbf_ndx
#define DBF_USED		api_function_address_table->api_dbf_used
#define	DBF_INDEXORDER		api_function_address_table->api_dbf_indexorder
#define	DBF_INDEXKEY		api_function_address_table->api_dbf_indexkey
#define DBF_SKIP		api_function_address_table->api_dbf_skip
#define DBF_GOTO		api_function_address_table->api_dbf_goto
#define DBF_SCATTER		api_function_address_table->api_dbf_scatter
#define DBF_GATHER		api_function_address_table->api_dbf_gather

/* memo functions */
#define MEMO_READ		api_function_address_table->api_memo_read
#define	MEMO_WRITE		api_function_address_table->api_memo_write
#define MEMO_UPDATE		api_function_address_table->api_memo_update
#define MEMO_RECLAIM		api_function_address_table->api_memo_reclaim
#define MEMO_MLINE		api_function_address_table->api_memo_mline
#define	MEMO_MLCOUNT		api_function_address_table->api_memo_mlcount
#define MEMO_SIZE		api_function_address_table->api_memo_size

/* blob functions */
#define	BLOB_READ		api_function_address_table->api_blob_read
#define BLOB_WRITE		api_function_address_table->api_blob_write
#define BLOB_UPDATE		api_function_address_table->api_blob_update
#define BLOB_RECLAIM		api_function_address_table->api_blob_reclaim
#define BLOB_SIZE		api_function_address_table->api_blob_size

/* memory variable functions */
#define	MEMVAR_LOOKUP		api_function_address_table->api_memvar_lookup
#define MEMVAR_DEFINE		api_function_address_table->api_memvar_define
#define MEMVAR_UPDATE		api_function_address_table->api_memvar_update

/* array variable functions */
#define ARRAY_LOOKUP		api_function_address_table->api_array_lookup
#define ARRAY_DEFINE		api_function_address_table->api_array_define
#define ARRAY_UPDATE		api_function_address_table->api_array_update
#define ARRAY_ALEN		api_function_address_table->api_array_alen

/* field variable functions */
#define FIELD_LOOKUP		api_function_address_table->api_field_lookup
#define	FIELD_NAME		api_function_address_table->api_field_name
#define FIELD_VALUE		api_function_address_table->api_field_value
#define	FIELD_COUNT		api_function_address_table->api_field_count
#define FIELD_UPDATE		api_function_address_table->api_field_update

/* date functions */
#define	DATE_CMONTH		api_function_address_table->api_date_cmonth
#define	DATE_CDOW		api_function_address_table->api_date_cdow
#define	DATE_CTOD		api_function_address_table->api_date_ctod
#define	DATE_DOW		api_function_address_table->api_date_dow
#define	DATE_DTOC		api_function_address_table->api_date_dtoc
#define	DATE_DAY		api_function_address_table->api_date_day
#define	DATE_MONTH		api_function_address_table->api_date_month
#define	DATE_YEAR		api_function_address_table->api_date_year
#define	DATE_DTOS		api_function_address_table->api_date_dtos
#define	DATE_STOD		api_function_address_table->api_date_stod
#define	DATE_TTOS		api_function_address_table->api_date_ttos
#define	DATE_STOT		api_function_address_table->api_date_stot
#define	DATE_DATE		api_function_address_table->api_date_date
#define	DATE_TIME		api_function_address_table->api_date_time
#define DATE_AMPM		api_function_address_table->api_date_ampm
#define	DATE_DATETIME		api_function_address_table->api_date_datetime

/* character string functions */
#define	CHAR_LTRIM		api_function_address_table->api_char_ltrim
#define	CHAR_RTRIM		api_function_address_table->api_char_rtrim
#define	CHAR_ALLTRIM		api_function_address_table->api_char_alltrim
#define	CHAR_LPAD		api_function_address_table->api_char_lpad
#define	CHAR_RPAD		api_function_address_table->api_char_rpad
#define	CHAR_UPPER		api_function_address_table->api_char_upper
#define	CHAR_LOWER		api_function_address_table->api_char_lower
#define CHAR_STR		api_function_address_table->api_char_str

/* currency string functions */
#define	CURR_STOY		api_function_address_table->api_date_stoy
#define	CURR_YTOS		api_function_address_table->api_date_ytos
/*=============================================================================
 * API level 4 functions
 *===========================================================================*/
	
#define	GETPROPERTY	    'G'
#define	SETPROPERTY	    'S'
#define METHODCALL	    'M'
#define COUNTPROPERTIES	    'C'+'P'
#define COUNTMETHODS	    'C'+'M'
#define COUNTEVENTS	    'C'+'E'
#define QUERYPROPERTIES	    'Q'+'P'
#define QUERYMETHODS	    'Q'+'M'
#define QUERYENEVENTS	    'Q'+'E'
#define UPDATEPROPERTIES    'U'
#define	_api_setobject_data	    api_function_address_table->api_setobject_data
#define	OBJECT_SETDATA(D)	    _api_setobject_data(_object, D);
#define	_api_getobject_data	    api_function_address_table->api_getobject_data
#define	OBJECT_GETDATA()	    _api_getobject_data(_object)
#define	_api_count_properties	    api_function_address_table->api_count_properties
#define OBJECT_COUNTPROPERTIES(E)   _api_count_properties(argv, E)
#define	_api_query_properties	    api_function_address_table->api_query_properties
#define OBJECT_QUERYPROPERTIES(E)   _api_query_properties(_object, E, argv[0],(char **)argv[1],(char **)argv[2])
#define	_api_count_methods	    api_function_address_table->api_count_methods
#define OBJECT_COUNTMETHODS(E)	    _api_count_methods(argv, E)
#define	_api_query_methods	    api_function_address_table->api_query_methods
#define OBJECT_QUERYMETHODS(E)	    _api_query_methods(_object, E, argv[0],(char **) argv[1],(char **)argv[2])
#define	_api_update_properties	    api_function_address_table->api_update_properties
#define OBJECT_UPDATEPROPERTIES()   _api_update_properties(_object, argv[0],(char **)argv[1],(char **)argv[2])
#define	_api_object_new		    api_function_address_table->api_object_new
#define	OBJECT_NEW		    _api_object_new
#define	_api_object_assign	    api_function_address_table->api_object_assign
#define	OBJECT_ASSIGN		    _api_object_assign
#define	_api_object_delete	    api_function_address_table->api_object_delete
#define	OBJECT_DELETE		    _api_object_delete
#define	_api_object_setproperty	    api_function_address_table->api_object_setproperty
#define	OBJECT_SETPROPERTY	    _api_object_setproperty
#define	_api_object_getproperty	    api_function_address_table->api_object_getproperty
#define	OBJECT_GETPROPERTY	    _api_object_getproperty
#define	_api_object_method	    api_function_address_table->api_object_method
#define	OBJECT_METHOD		    _api_object_method
#define	_api_object_getarg	    api_function_address_table->api_object_getarg
#define	OBJECT_GETARG		    _api_object_getarg
#define	_api_object_setarg	    api_function_address_table->api_object_setarg
#define	OBJECT_SETARG		    _api_object_setarg
#ifdef _WINDOWS
#define	DEFINE_CLASS(CLASSNAME) \
    CLASSNAME(char	*_object,char	*message,int	argc,char	*argv[])
#define	DEFINE_METHOD(CLASSNAME, METHODNAME) \
    CLASSNAME##METHODNAME(char	*_object,int argc,char	*argv[])
#define	DEFINE_PROPERTYGET(CLASSNAME, PROPERTYNAME) \
    CLASSNAME##PROPERTYNAME##Get(char	*_object,char	*_propertyValue)
#define	DEFINE_PROPERTYSET(CLASSNAME, PROPERTYNAME) \
    CLASSNAME##PROPERTYNAME##Set(char	*_object,char	*_propertyName,char	*_propertyValue)
#define DISPATCH_METHOD(CLASSNAME, METHODNAME) \
	if (*message == 'M' && stricmp(argv[0], #METHODNAME) == 0) { \
	    return(CLASSNAME##METHODNAME(_object, argc, argv)); \
	} 
#define DISPATCH_PROPGET(CLASSNAME, PROPERTYNAME) \
	if (*message == 'G' && stricmp(argv[0], #PROPERTYNAME) == 0) { \
	    return(CLASSNAME##PROPERTYNAME##Get(_object, argv[1])); \
	}
#define DISPATCH_PROPSET(CLASSNAME, PROPERTYNAME) \
	if (*message == 'S' && stricmp(argv[0], #PROPERTYNAME) == 0) { \
	    return(CLASSNAME##PROPERTYNAME##Set(_object, #PROPERTYNAME, argv[1])); \
	}
#else
#define	DEFINE_CLASS(CLASSNAME) \
    CLASSNAME(_object, message, argc, argv)\
    char	*_object; \
    char	*message; \
    int		argc; \
    char	*argv[];
#define	DEFINE_METHOD(CLASSNAME, METHODNAME) \
    CLASSNAME##METHODNAME(_object, argc, argv)\
    char	*_object; \
    int		argc; \
    char	*argv[];
#define	DEFINE_PROPERTYGET(CLASSNAME, PROPERTYNAME) \
    CLASSNAME##PROPERTYNAME##Get(_object, _propertyValue)\
    char	*_object; \
    char	*_propertyValue;
#define	DEFINE_PROPERTYSET(CLASSNAME, PROPERTYNAME) \
    CLASSNAME##PROPERTYNAME##Set(_object, _propertyName, _propertyValue)\
    char	*_object; \
    char	*_propertyName; \
    char	*_propertyValue;
#define DISPATCH_METHOD(CLASSNAME, METHODNAME) \
	if (*message == 'M' && strcasecmp(argv[0], #METHODNAME) == 0) { \
	    return(CLASSNAME##METHODNAME(_object, argc, argv)); \
	} 
#define DISPATCH_PROPGET(CLASSNAME, PROPERTYNAME) \
	if (*message == 'G' && strcasecmp(argv[0], #PROPERTYNAME) == 0) { \
	    return(CLASSNAME##PROPERTYNAME##Get(_object, argv[1])); \
	}
#define DISPATCH_PROPSET(CLASSNAME, PROPERTYNAME) \
	if (*message == 'S' && strcasecmp(argv[0], #PROPERTYNAME) == 0) { \
	    return(CLASSNAME##PROPERTYNAME##Set(_object, #PROPERTYNAME, argv[1])); \
	}
#endif
#define DISPATCH_FACTORY() \
        switch ((*message != 'C' && *message != 'Q') ? *message : (message[0] + message[5])) { \
	case COUNTPROPERTIES: \
	    OBJECT_COUNTPROPERTIES(puplic_properties); \
	    return(0); \
	case QUERYPROPERTIES: \
	    OBJECT_QUERYPROPERTIES(puplic_properties); \
	    return(0); \
	case COUNTMETHODS: \
	    OBJECT_COUNTMETHODS(public_methods); \
	    return(0); \
	case QUERYMETHODS: \
	    OBJECT_QUERYMETHODS(public_methods); \
             return(0); \
	case UPDATEPROPERTIES: \
	    OBJECT_UPDATEPROPERTIES(); \
	    return(0) ;\
	}
#define	OBJECT_RETERROR(M)	\
	(sprintf(argv[0], "E:%s", M)); \
	return(-1);
#define OBJECT_RETRESULT(R) \
	OBJECT_SETARG(argv[1], R); \
	return((int)(argv[1]))
#define	OBJECT_RETPROPERTY(T, V) \
    switch (T) { \
	case 'N': \
	    sprintf(_propertyValue, "N:%f", V); \
	    break; \
	case 'L': \
	    sprintf(_propertyValue, "L:%c", V); \
	    break; \
	case 'D': \
	    sprintf(_propertyValue, "D:%s", V); \
	    break; \
	case 'T': \
	    sprintf(_propertyValue, "T:%s", V); \
	    break; \
	case 'Y': \
	    sprintf(_propertyValue, "Y:%s", V); \
	    break; \
	case 'C': \
	    sprintf(_propertyValue, "C:%s", V); \
	    break; \
	case 'O': \
	    sprintf(_propertyValue, "O:%ld", V); \
	    break; \
	} \
	return((int) _propertyValue) ;
#define	OBJECT_GETARGC()	    (argc-2)
#define	OBJECT_GETPARAMETER(N, R)	    OBJECT_GETARG((argv[N+1]), R);
#define	OBJECT_GETVALUE(R)		    OBJECT_GETARG(_propertyValue, R);
#define	OBJECT_GETTYPE()		    _propertyValue[0]
#define OBJECT_ERROR			    -2;
#define OBJECT_SUCCESS			    0;
#define OBJECT_GETOBJECT()		    (OBJECT)atol(_propertyValue+2);


/*----------------------------*/
/* API function address table */
/*----------------------------*/
struct	API_FUNCTION_ADDRESS_TABLE {
	/*=============================================================================
	 * API level 1 functions 
	 *===========================================================================*/
	char	*(*api_par_c)(int, ...);
	char	*(*api_par_ds)(int, ...);
	char	*(*api_par_ts)(int, ...);
	char	*(*api_par_ys)(int, ...);
	int	(*api_par_l)(int, ...);	
	int	(*api_par_ni)(int, ...);
	int	(*api_par_nl)(int, ...);	
	double	(*api_par_nd)(int, ...);
	int	(*api_par_clen)(int, ...);	
	int	(*api_par_csiz)(int, ...);	
	int	(*api_par_infa)(int, ...);	
	int	(*api_par_info)(int);	
	OBJECT	(*api_par_o)(int, ...);
	void	(*api_ret_c)(char *);	
	void	(*api_ret_clen)(char *, int);	
	void	(*api_ret_ds)(char *);	
	void	(*api_ret_ts)(char *);	
	void	(*api_ret_ys)(char *);	
	void	(*api_ret_l)(int);	
	void	(*api_ret_ni)(int);	
	void	(*api_ret_nl)(int);
	void	(*api_ret_nd)(double);	
	void	(*api_ret_o)(OBJECT);	
	void	(*api_ret)();
	int	(*api_parse_parameters)(int, char *, ...);		/* new in 9.0.4 */

	/*=============================================================================
	 * API level 2 functions 
	 *===========================================================================*/
	int	(*api_command)(char *);
	int	(*api_expression)(struct API_EXPRESSION *, char*);
	void	(*api_error)(int, char *);
	/*=============================================================================
	 * API level 3 functions 
	 *===========================================================================*/
	int	(*api_dbf_fetch)(int, char*,char*,int,int,int);
	int	(*api_dbf_read)(long, int, int);
	int	(*api_dbf_recall)();
	int	(*api_dbf_delete)();
	int	(*api_dbf_update)();
	int	(*api_dbf_append)(int);
	int	(*api_dbf_seek)(char*, int);
	int	(*api_dbf_lockf)();
	int	(*api_dbf_unlockf)();
	int	(*api_dbf_lockr)(int);
	int	(*api_dbf_unlockr)(int);
	int	(*api_dbf_scatter)(char *);
	int	(*api_dbf_gather)(char *);
	int	(*api_dbf_goto)(int);
	int	(*api_dbf_skip)(int);
	char	*(*api_dbf_recbuffer)();
	char	*(*api_dbf_alias)();
	char	*(*api_dbf_dbf)();
	char	*(*api_dbf_filter)();
	char	*(*api_dbf_fmt)();
	int	(*api_dbf_isexclusive)();
	int	(*api_dbf_isread_only)();
	char	*(*api_dbf_ndx)(int);
	int	(*api_dbf_reccount)();
	int	(*api_dbf_recsize)();
	int	(*api_dbf_recno)();
	int	(*api_dbf_select)(int);
	int	(*api_dbf_used)();
	int	(*api_dbf_isdeleted)();
	int	(*api_dbf_isbof)();
	int	(*api_dbf_iseof)();
	int	(*api_dbf_indexorder)();
	char	*(*api_dbf_indexkey)(int);
	int	(*api_memo_read)(char *, char*, int);
	int	(*api_memo_write)(char *, char *);
	int	(*api_memo_update)(char *, char*);
	int	(*api_memo_reclaim)(char *);
	char	*(*api_memo_mline)(char*, char *, int);
	int	(*api_memo_mlcount)(char *);
	int	(*api_memo_size)(char*);
	int	(*api_blob_read)(char*, unsigned char*, int);
	int	(*api_blob_write)(char*, unsigned char*, int);
	int	(*api_blob_update)(char*, unsigned char*, int);
	int	(*api_blob_reclaim)(char *);
	int	(*api_blob_size)(char *);
	struct API_MEMVAR	*(*api_memvar_lookup)(char *);
	struct API_MEMVAR	*(*api_memvar_define)(char *, int);
	int	(*api_memvar_update)(char *, char, int, int, char *, double *, char, DATE, DATETIME *, CURRENCY *);
	struct API_MEMVAR	*(*api_array_lookup)(char *, int);
	char	*(*api_array_define)(char *, int, int, int);
	int	(*api_array_update)(char *, int, char, int, int, char *, double *, char, DATE);
	int	(*api_array_alen)(char *);
	struct API_FIELD	*(*api_field_lookup)(char *, struct API_FIELD *);
	char	*(*api_field_name)(int);
	int	(*api_field_value)(char *, char *);
	int	(*api_field_update)(char *, char, char *, double *,char, DATE);
	int	(*api_field_count)();
	char	*(*api_date_cmonth)(DATE);
	char	*(*api_date_cdow)(DATE);
	DATE	(*api_date_ctod)(char *);
	int	(*api_date_dow)(DATE);
	char	*(*api_date_dtoc)(DATE);
	int	(*api_date_day)(DATE);
	int	(*api_date_month)(DATE);
	int	(*api_date_year)(DATE);
	char	*(*api_date_dtos)(DATE);
	DATE	(*api_date_stod)(char *);
	char	*(*api_date_ttos)(DATETIME);
	DATETIME*(*api_date_stot)(char *);
	char	*(*api_date_date)();
	char	*(*api_date_time)();
	char	*(*api_date_ampm)();
	char	*(*api_date_datetime)();
	char	*(*api_char_ltrim)(char *);
	char	*(*api_char_rtrim)(char *);
	char	*(*api_char_alltrim)(char *);
	char	*(*api_char_lpad)(char *, char *, int, char);
	char	*(*api_char_rpad)(char *, char *, int, char);
	char	*(*api_char_upper)(char *);
	char	*(*api_char_lower)(char *);
	char	*(*api_char_str)(char *, double, int, int);
	char	*(*api_date_ytos)(CURRENCY);
	CURRENCY*(*api_date_stoy)(char *);
	/*=============================================================================
	 * API level 4 functions 
	 *===========================================================================*/
	void	(*api_setobject_data)(OBJECT object, char *data);
	char	*(*api_getobject_data)(OBJECT object);
	void	(*api_count_properties)(char	*argv[],char	**properties);
	int	(*api_query_properties)(OBJECT	object,
					char    **local_properties,
					char    *property_argc,
					char    **prop_names,    
					char    **prop_desc);  
	void	(*api_count_methods)(char	*argv[],char	**methods);
	int	(*api_query_methods)(OBJECT	object,
					char    **local_methods,
					char    *method_argc,
					char    **meth_names,    
					char    **meth_desc);  
	int	(*api_update_properties)(OBJECT	object,
					char    *property_argc,
					char    **prop_names,    
					char    **prop_values);
	OBJECT	(*api_object_new)(char	*object_name, char   *class_name, char    *parameters);
	OBJECT	(*api_object_assign)(OBJECT   objptr, char   *object_name);
	void	(*api_object_delete)(OBJECT   objptr);
	int	(*api_object_setproperty)(OBJECT object, char *property_name, char	*value);
	OBJARG	(*api_object_getproperty)(OBJECT object, char *property_name, char	*buffer);
	OBJARG	(*api_object_method)(OBJECT object, char *method_name, char	*args);
	int	(*api_object_getarg)(OBJARG objarg, struct API_EXPRESSION *buffer);
	OBJARG	(*api_object_setarg)(OBJARG objarg, struct API_EXPRESSION *buffer);
};

#endif
